<?php

namespace app\admin\controller\Products;

use app\BaseController;
use app\common\model\Products\Product as ModelProduct;
use app\Request;
use think\Validate;
use think\exception\ValidateException;
use think\facade\Filesystem;
use app\common\arw\adjfut\src\Excel;
use app\common\arw\adjfut\src\UploadFile;
use app\common\exception\Tool;
use think\facade\Db;
use think\facade\Env;


class Product extends BaseController
{
    /**
     * 获取产品列表
     */
    public function getProductList(Request $request, $isExport = false): array
    {
        $params = $request->param();
        $con = [];

        $con = Tool::getOptionalQuery(['a.product_type_guid', '='], ['product_name', 'LIKE'],);

        $query = ModelProduct::alias('a')
            ->leftjoin('product_type b', 'a.product_type_guid = b.product_type_guid')
            ->where($con)
            ->field([
                'a.product_id',
                'a.product_guid',
                'a.product_type_guid',
                'b.product_type_name',
                'a.product_name',
                'a.product_img',
                'a.product_description',
                'a.product_link',
                'a.product_details',
                'a.product_sort'
            ])
            ->order('product_sort', 'asc');

        return $isExport ?  $query->select()->toArray() : msg("获取产品列表成功!", $query);
    }

    /**
     * 添加产品
     */
    public function addProduct(Request $request): array
    {
        $params = $request->param();
        $this->validate($params, [
            'product_type_guid|产品类目' => 'require',
            'product_name|产品名称' => 'require',
            'product_img|产品图片' => 'require',
            'product_sort|产品排序' => 'require'
        ]);
        $model = ModelProduct::create($params, [
            'product_type_guid',
            'product_name',
            'product_img',
            'product_description',
            'product_link',
            'product_details',
            'product_sort',
            'product_guid',
            'product_create_user_guid',
            'product_update_user_guid'
        ]);
        return msg('添加成功!');
    }

    /**
     * 编辑产品
     */
    public function editProduct(Request $request): array
    {
        $params = $request->param();
        $this->validate($params, [
            'product_type_guid|产品类目' => 'require',
            'product_name|产品名称' => 'require',
            'product_img|产品图片' => 'require',
            'product_sort|产品排序' => 'require'
        ]);
        $model = ModelProduct::where('product_guid', $params['product_guid'])->find();
        if (!$model) throwErrorMsg("该产品不存在", 1);
        $model->allowField([
            'product_type_guid',
            'product_name',
            'product_img',
            'product_description',
            'product_link',
            'product_details',
            'product_sort',
            'product_update_user_guid'
        ])->save($params);
        return msg('编辑成功!');
    }

    /**
     * 删除产品
     */
    public function deleteProduct(Request $request): array
    {
        $params = $request->param();
        $this->validate($params, [
            'product_guid' => 'require',
        ]);
        $product = ModelProduct::where([
            'product_guid' => explode(',', $params['product_guid'])
        ])->select();
        $product->delete();
        return msg('删除成功!');
    }

    /**
     * 导出Excel
     */
    public function exportExcel(Request $request): void
    {
        ModelProduct::exportExcel(self::getProductList($request, true));
    }

    /**
     * 下载导入模板
     */
    public function downloadTemplate(Request $request): void
    {
        $params = $request->param();
        $data = [
            array_values(ModelProduct::EXCELFIELD),
            [
                '软件工具',
                'Arm Development StudioArm',
                'http://localhost:3000/uploads/ProductImg/20230628/f5a7065c3db295c1e3e2a26b36cb41ed.jpg',
                'Arm全面端到端的嵌入式开发工具',
                '',
                '',
                '1',
            ]
        ];
        $excel = (new Excel())->exporTsheet($data);
        $excel->save('产品导入模板.xlsx');
    }

    /**
     * 导入excel
     */
    public function importExcel(Request $request): array
    {
        $file = new UploadFile('uploads', 'fileExt:xlsx');
        $file->putFile('product');
        $msg = ModelProduct::importExcel($file);
        return [
            'code' => 0,
            'msg' => $msg
        ];
    }
}
