<?php

namespace app\common\model\Products;

use app\common\arw\adjfut\src\Validate;
use app\BaseModel;
use think\model\concern\SoftDelete;
use app\common\arw\adjfut\src\Excel;
use app\Request;
use app\common\exception\Tool;
use think\facade\Db;
use app\common\model\Products\ProductType as ModelProductType;

class Product extends BaseModel
{
    use SoftDelete;
    // 删除字段
    protected $deleteTime = 'product_delete_time';
    // 设置主键名
    protected $pk = 'product_guid';
    // 设置废弃字段
    protected $disuse = [];
    // 设置字段信息
    protected $schema = [
        'product_id' => 'int',
        'product_guid' => 'string',
        'product_type_guid' => 'string',
        'product_name' => 'string',
        'product_img' => 'string',
        'product_description' => 'string',
        'product_link' => 'string',
        'product_details' => '',
        'product_sort' => 'int',
        'product_create_time' => 'datetime',
        'product_create_user_guid' => 'string',
        'product_update_time' => 'datetime',
        'product_update_user_guid' => 'string',
        'product_delete_time' => 'datetime',
        'product_delete_user_guid' => 'string',

    ];
    // 设置json类型字段
    protected $json = [''];
    // 开启自动写入时间戳字段
    protected $autoWriteTimestamp = 'datetime';
    // 创建时间
    protected $createTime = 'product_create_time';
    // 修改时间
    protected $updateTime = 'product_update_time';

    // excel导入/下载模板表头
    public const EXCELFIELD = [
        'product_type_guid' => '产品类目',
        'product_name' => '产品名称',
        'product_img' => '产品图片',
        'product_description' => '产品描述',
        'product_link' => '产品跳转链接',
        'product_details' => '产品详情',
        'product_sort' => '产品排序',
    ];

    /**
     * 新增前
     */
    public static function onBeforeInsert(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeCreateField();
    }

    /**
     * 更新前
     */
    public static function onBeforeUpdate(self $model): void
    {
        // self::checkRepeatData($model);
        $model->completeUpdateField();
    }

    /**
     * 删除前
     */
    public static function onBeforeDelete(self $model): void
    {
        $model->completeDeleteField();
    }


    /**
     * 导出Excel
     * 
     * @param array $select 导出的数据
     */
    public static function exportExcel(array $select): void
    {
        $data = [[
            '产品类目',
            '产品名称',
            '产品图片',
            '产品描述',
            '产品跳转链接',
            '产品排序'
        ]];
        foreach ($select as $key => $val) {

            $product_type = ModelProductType::where('product_type_guid',$val['product_type_guid'])->find();
            if(!$product_type) throwErrorMsg("产品类目:【{$val['product_type_guid']}】不存在");
            $product_type_name = $product_type->product_type_name;

            $data[] = [
                $product_type_name,
                $val['product_name'],
                Excel::ExportImgFiled($val['product_img']),
                $val['product_description'],
                $val['product_link'],
                $val['product_sort'],
            ];
        }
        $excel = (new Excel())->exporTsheet($data);
        $excel->save('产品.xlsx');
    }


    /**
     * 导入excel
     * 
     * @param \app\common\arw\adjfut\src\UploadFile $file excel
     */
    public static function importExcel(\app\common\arw\adjfut\src\UploadFile $file): string
    {
        $msg = [];

        Db::startTrans();
        try {
            $excel = new Excel($file);
            $data = $excel->parseExcel(
                Tool::getExcelRule(self::EXCELFIELD),
                ['titleLine' => [1]]
            );
            if (!$data)  throwErrorMsg('excel无数据', 1);
            $msg = [];
            foreach ($data as $line => $value) {
                try {
                    $model = self::importExcelInit($value);
                    $msg[] = "{$line} <span style='color:#27af49'>商品:【{$value['product_name']}】新增成功!</span><br>";
                } catch (\Throwable $th) {
                    $msg[] = "{$line} <span style='color:red'>{$th->getMessage()}</span><br>";
                }
            }
            Db::commit();
            return implode(', ', $msg);
        } catch (\Throwable $th) {
            Db::rollback();
            throw $th;
        }
    }


    /**
     * 导入excel初始化
     * 
     * @param array $value excel每行数据
     */
    public static function importExcelInit(array $value): void
    {
        $product_type_guid = $value['product_type_guid'];
        $product_name = $value['product_name'];
        $product_img = $value['product_img'];
        $product_description = $value['product_description'];
        $product_link = $value['product_link'];
        $product_details = $value['product_details'];
        $product_sort = $value['product_sort'];

        $product_type = ModelProductType::where('product_type_name',$product_type_guid)->find();
        if(!$product_type) throwErrorMsg("产品类目:【{$product_type_guid}】不存在");
        $product_type_guid = $product_type->product_type_guid;

        self::create(
            [
                'product_type_guid' => $product_type_guid,
                'product_name' => $product_name,
                'product_img' => $product_img,
                'product_description' => $product_description,
                'product_link' => $product_link,
                'product_details' => $product_details,
                'product_sort' => $product_sort,
            ],
            [
                'product_type_guid',
                'product_name',
                'product_img',
                'product_description',
                'product_link',
                'product_details',
                'product_sort',
                'product_guid',
                'product_create_user_guid',
                'product_update_user_guid'
            ]
        );
    }
}
